// *********************************************************************************
// INCLUDES
// *********************************************************************************
#include <sourcemod>
#include <sdktools>
#include <adt>
#include <tf2>
#include <tf2_stocks>
#include <items_core>
#include <clientprefs>

// *********************************************************************************
// PREPROCESSOR
// *********************************************************************************
#pragma semicolon 1                  // Force strict semicolon mode.

// *********************************************************************************
// CONSTANTS
// *********************************************************************************
#define PLUGIN_NAME         "[TF2] Wearable Items [Manager]"
#define PLUGIN_AUTHOR       "Damizean"
#define PLUGIN_VERSION      "1.0"
#define PLUGIN_CONTACT      "elgigantedeyeso@gmail.com"
#define CVAR_FLAGS          FCVAR_PLUGIN|FCVAR_NOTIFY

#define PLUGIN_LEVEL        10

#define FLAG_CLASS_SCOUT           (1 << 0)
#define FLAG_CLASS_SNIPER          (1 << 1)
#define FLAG_CLASS_SOLDIER         (1 << 2)
#define FLAG_CLASS_DEMOMAN         (1 << 3)
#define FLAG_CLASS_MEDIC           (1 << 4)
#define FLAG_CLASS_HEAVY           (1 << 5)
#define FLAG_CLASS_PYRO            (1 << 6)
#define FLAG_CLASS_SPY             (1 << 7)
#define FLAG_CLASS_ENGINEER        (1 << 8)
#define FLAG_CLASS_ALL             (0b111111111)

#define FLAG_TEAM_RED              (1 << 0)
#define FLAG_TEAM_BLU              (1 << 1)

enum eManagerCode
{
    Manager_Success                = 1,
    ManagerError_FileNotFound      = -1,
    ManagerError_ParseError        = -2
};

// *********************************************************************************
// VARIABLES
// *********************************************************************************
new Handle:g_hGlobal;
new Handle:g_hParsedTrie;
new Handle:g_hParser;
new bool:g_bParsingItem;

new Handle:g_hClientPrefs[10][MAX_SLOTS];

new Handle:g_hMenuMain;
new Handle:g_hMenuSlots[MAX_SLOTS];

// *********************************************************************************
// PLUGIN
// *********************************************************************************
public Plugin:myinfo =
{
    name        = PLUGIN_NAME,
    author      = PLUGIN_AUTHOR,
    description = PLUGIN_NAME,
    version     = PLUGIN_VERSION,
    url         = PLUGIN_CONTACT
};

// *********************************************************************************
// METHODS
// *********************************************************************************

/* OnPluginStart()
**
** When the plugin is loaded.
** -------------------------------------------------------------------------- */
public OnPluginStart()
{
    // Check if the plugin is being run on the proper mod.
    decl String:strModName[32]; GetGameFolderName(strModName, sizeof(strModName));
    if (!StrEqual(strModName, "tf")) SetFailState("This plugin is only for Team Fortress 2.");
    
    // Create plugin cvars
    CreateConVar("tf_items_manager_version", PLUGIN_VERSION, PLUGIN_NAME, FCVAR_PLUGIN|FCVAR_NOTIFY);
    
    // Init everything else
    ParseConfiguration();
    StartupClientPrefs();
    StartupCommands();
    StartupEvents();
}

/* OnMapStart()
**
** At the start of a map.
** -------------------------------------------------------------------------- */
public OnMapStart()
{
    g_hMenuMain = MenuCreateMain();
    for (new iSlot = 0; iSlot < 8; iSlot++) g_hMenuSlots[iSlot] = MenuCreateSlot(iSlot);
}

/* OnMapEnd()
**
** At the end of a map.
** -------------------------------------------------------------------------- */
public OnMapEnd()
{
    if (g_hMenuMain  != INVALID_HANDLE) { CloseHandle(g_hMenuMain);  g_hMenuMain  = INVALID_HANDLE; }
    for (new iSlot = 0; iSlot < 8; iSlot++) 
    {
        if (g_hMenuSlots[iSlot] != INVALID_HANDLE)
        {
            CloseHandle(g_hMenuSlots[iSlot]);
            g_hMenuSlots[iSlot] = INVALID_HANDLE;
        }
    }
}

/* StartupClientPrefs()
**
** 
** -------------------------------------------------------------------------- */
StartupClientPrefs()
{
    // Create clientprefs
    for (new iClass = 0; iClass < 10; iClass++)
    for (new iSlot = 0; iSlot < MAX_SLOTS; iSlot++)
    {
        decl String:strCookie[64];
        Format(strCookie, sizeof(strCookie), "tf_items_%i", (iClass*MAX_SLOTS)+iSlot);
        g_hClientPrefs[iClass][iSlot] = RegClientCookie(strCookie, "", CookieAccess_Private);
    }
}

/* StartupCommands()
**
** 
** -------------------------------------------------------------------------- */
StartupCommands()
{
    RegConsoleCmd("tf_items",      Cmd_Items);
    RegConsoleCmd("items",         Cmd_Items);
    RegConsoleCmd("hats",          Cmd_Items);
    RegConsoleCmd("em",            Cmd_Items);
    RegConsoleCmd("say",           Cmd_BlockTriggers);
    RegConsoleCmd("say_team",      Cmd_BlockTriggers);
}

/* StartupEvents()
**
** 
** -------------------------------------------------------------------------- */
StartupEvents()
{
    HookEvent("player_changeclass", EventChangeClass);
}

/*
**
**     __  ___                                                  __ 
**    /  |/  /___ _____  ____ _____ ____  ____ ___  ___  ____  / /_
**   / /|_/ / __ `/ __ \/ __ `/ __ `/ _ \/ __ `__ \/ _ \/ __ \/ __/
**  / /  / / /_/ / / / / /_/ / /_/ /  __/ / / / / /  __/ / / / /_  
** /_/  /_/\__,_/_/ /_/\__,_/\__, /\___/_/ /_/ /_/\___/_/ /_/\__/  
**                          /____/                                 
**
*/

/* EventChangeClass()
**
** Whenever a client has changed it's class.
** -------------------------------------------------------------------------- */
public EventChangeClass(Handle:hEvent, String:strName[], bool:bDontBroadcast)
{
    new iClient = GetClientOfUserId(GetEventInt(hEvent, "userid"));
    new iClass  = GetEventInt(hEvent, "class");
    
    if (IsValidClient(iClient))
        ReadClientPrefs(iClient, iClass);
}

/* OnItemCreated()
**
** Called whenever an item has been created and registered in the plugin.
** -------------------------------------------------------------------------- */
public OnItemCreated(Handle:hItem)
{
    decl String:strBuffer[256];
    decl String:strTokens[32][256];
    new iBodyGroups       = 0;
    new iClassRestriction = 0;
    new iTeamRestriction  = 0;
    new iAdminRestriction = 0;
    
    // Find the proper body group flags for this hat
    if (GetTrieString(hItem, "bodygroups-table", strBuffer, sizeof(strBuffer)))
    {
        new iNumStrings = ExplodeString(strBuffer, ",", strTokens, 32, 256);
        for (new iToken = 0; iToken < iNumStrings; iToken++)
        {
            TrimString(strTokens[iToken]);
            if (StrEqual(strTokens[iToken], "scout_hat"))             iBodyGroups |= FLAG_HIDE_SCOUT_HAT;
            else if (StrEqual(strTokens[iToken], "scout_headphones")) iBodyGroups |= FLAG_HIDE_SCOUT_HEADPHONES;
            else if (StrEqual(strTokens[iToken], "heavy_hands"))      iBodyGroups |= FLAG_HIDE_HEAVY_HANDS;
            else if (StrEqual(strTokens[iToken], "engineer_helmet"))  iBodyGroups |= FLAG_HIDE_ENGINEER_HELMET;
            else if (StrEqual(strTokens[iToken], "sniper_quiver"))    iBodyGroups |= FLAG_SHOW_SNIPER_QUIVER;
            else if (StrEqual(strTokens[iToken], "sniper_hat"))       iBodyGroups |= FLAG_HIDE_SNIPER_HAT;
            else if (StrEqual(strTokens[iToken], "soldier_rocket"))   iBodyGroups |= FLAG_HIDE_SOLDIER_ROCKET;
            else if (StrEqual(strTokens[iToken], "soldier_medal"))    iBodyGroups |= FLAG_SHOW_SOLDIER_MEDAL;
            else if (StrEqual(strTokens[iToken], "soldier_helmet"))   iBodyGroups |= FLAG_HIDE_SOLDIER_HELMET;
        }
    }
    SetTrieValue(hItem, "bodygroups", iBodyGroups);
    
    // Find the class restriction key and set the proper flag values
    if (GetTrieString(hItem, "class-restrictions", strBuffer, sizeof(strBuffer)))
    {
        new iNumStrings = ExplodeString(strBuffer, ",", strTokens, 32, 256);
        for (new iToken = 0; iToken < iNumStrings; iToken++)
        {
            TrimString(strTokens[iToken]);
            if (StrEqual(strTokens[iToken], "all"))           iClassRestriction |= FLAG_CLASS_ALL;
            else if (StrEqual(strTokens[iToken], "scout"))    iClassRestriction |= FLAG_CLASS_SCOUT;
            else if (StrEqual(strTokens[iToken], "sniper"))   iClassRestriction |= FLAG_CLASS_SNIPER;
            else if (StrEqual(strTokens[iToken], "soldier"))  iClassRestriction |= FLAG_CLASS_SOLDIER;
            else if (StrEqual(strTokens[iToken], "demoman"))  iClassRestriction |= FLAG_CLASS_DEMOMAN;
            else if (StrEqual(strTokens[iToken], "medic"))    iClassRestriction |= FLAG_CLASS_MEDIC;
            else if (StrEqual(strTokens[iToken], "heavy"))    iClassRestriction |= FLAG_CLASS_HEAVY;
            else if (StrEqual(strTokens[iToken], "pyro"))     iClassRestriction |= FLAG_CLASS_PYRO;
            else if (StrEqual(strTokens[iToken], "spy"))      iClassRestriction |= FLAG_CLASS_SPY;
            else if (StrEqual(strTokens[iToken], "engineer")) iClassRestriction |= FLAG_CLASS_ENGINEER;
        }
    }
    SetTrieValue(hItem, "class-restrictions", iClassRestriction);
    
    // Find the team restriction key and set the proper flag values
    if (GetTrieString(hItem, "team-restrictions", strBuffer, sizeof(strBuffer)))
    {
        new iNumStrings = ExplodeString(strBuffer, ",", strTokens, 32, 256);
        for (new iToken = 0; iToken < iNumStrings; iToken++)
        {
            TrimString(strTokens[iToken]);
            if (StrEqual(strTokens[iToken], "all"))       iTeamRestriction |= FLAG_TEAM_BLU|FLAG_TEAM_RED;
            else if (StrEqual(strTokens[iToken], "blue")) iTeamRestriction |= FLAG_TEAM_BLU;
            else if (StrEqual(strTokens[iToken], "red"))  iTeamRestriction |= FLAG_TEAM_RED;
        }
    }
    SetTrieValue(hItem, "team-restrictions", iTeamRestriction);
    
    // Find the admin restriction key and set the proper flag values
    if (GetTrieString(hItem, "admin-restrictions", strBuffer, sizeof(strBuffer)))
    {
        iAdminRestriction = ReadFlagString(strBuffer);
    }
    SetTrieValue(hItem, "admin-restrictions", iAdminRestriction);
}

/* CanPlayerUseItem()
**
** Determines if the given item can be used by the client. The reference va-
** riable "bCanUse" tells the plugin if it's permitted to use it or not. The
** "iTopLevel" one is an optional variable in case you want to stablish an
** hierarchy between plugins when deciding if it's usable.
**
** For example, if plugin A with level 4 says the item is usable, but the
** plugin B with level 10 says it isn't, if both plugins respect the hierachical
** levels, the plugin B will be the one to decide if it's useable.
** -------------------------------------------------------------------------- */
public CanPlayerUseItem(iClient, Handle:hItem, &bool:bCanUse, &iTopLevel)
{
    new iClassRestriction; GetTrieValue(hItem, "class-restrictions", iClassRestriction);
    new iTeamRestriction;  GetTrieValue(hItem, "team-restrictions",  iTeamRestriction);
    new iAdminRestriction; GetTrieValue(hItem, "admin-restrictions", iAdminRestriction);
    // Check if a superior hierarchy already decided the usage..
    if (iTopLevel > PLUGIN_LEVEL) return;
    iTopLevel = PLUGIN_LEVEL;
    
    // Determine if it's possible by class
    switch(TF2_GetPlayerClass(iClient))
    {
        case TFClass_Unknown: bCanUse = false;
        case TFClass_Scout:    if (!(iClassRestriction & FLAG_CLASS_SCOUT))    { bCanUse = false; return; }
        case TFClass_Sniper:   if (!(iClassRestriction & FLAG_CLASS_SNIPER))   { bCanUse = false; return; }
        case TFClass_Soldier:  if (!(iClassRestriction & FLAG_CLASS_SOLDIER))  { bCanUse = false; return; }
        case TFClass_DemoMan:  if (!(iClassRestriction & FLAG_CLASS_DEMOMAN))  { bCanUse = false; return; }
        case TFClass_Medic:    if (!(iClassRestriction & FLAG_CLASS_MEDIC))    { bCanUse = false; return; }
        case TFClass_Heavy:    if (!(iClassRestriction & FLAG_CLASS_HEAVY))    { bCanUse = false; return; }
        case TFClass_Pyro:     if (!(iClassRestriction & FLAG_CLASS_PYRO))     { bCanUse = false; return; }
        case TFClass_Spy:      if (!(iClassRestriction & FLAG_CLASS_SPY))      { bCanUse = false; return; }
        case TFClass_Engineer: if (!(iClassRestriction & FLAG_CLASS_ENGINEER)) { bCanUse = false; return; }
    }
    
    // Determine if it's possible by team
    switch(GetClientTeam(iClient))
    {
        case (_:TFTeam_Unassigned): { bCanUse = false; return; }
        case (_:TFTeam_Spectator):  { bCanUse = false; return; } 
        case (_:TFTeam_Red):        if (!(iTeamRestriction & FLAG_TEAM_RED)) { bCanUse = false; return; }
        case (_:TFTeam_Blue):       if (!(iTeamRestriction & FLAG_TEAM_BLU)) { bCanUse = false; return; } 
    }
    
    // Determine if it's possible by admin
    if (iAdminRestriction != 0)
    {
        if (!(GetUserFlagBits(iClient) & iAdminRestriction)) { bCanUse = false; return; }
    }
}

/* OnPlayerEquipItem()
**
** Called whenever a client successfully equipped an item.
** -------------------------------------------------------------------------- */
public OnPlayerEquipItem(iClient, Handle:hItem, bool:bRegenerating)
{
    if (bRegenerating == false)
    {
        WriteClientPrefs(iClient, GetItemSlot(hItem), hItem);
    }
}

/* OnPlayerRemoveItem()
**
** Called whenever a client successfully removed an item.
** -------------------------------------------------------------------------- */
public OnPlayerRemoveItem(iClient, Handle:hItem, bool:bDestroying)
{
    if (bDestroying == false)
    {
        WriteClientPrefs(iClient, GetItemSlot(hItem), INVALID_HANDLE);
    }
}



/* ReadClientPrefs()
**
** Retrieves the client preferences for this user and sets the proper hats.
** -------------------------------------------------------------------------- */
ReadClientPrefs(iClient, iClass)
{
    decl String:strName[64];
    decl Handle:hItem;
            
    if (AreClientCookiesCached(iClient))
    {
        for (new iSlot = 0; iSlot < MAX_SLOTS; iSlot++)
        {
            GetClientCookie(iClient, g_hClientPrefs[iClass][iSlot], strName, sizeof(strName));
            hItem = FindItemByName(strName);
            if (hItem != INVALID_HANDLE)
            {
                if (GetItemSlot(hItem) == iSlot) SetItemAtSlot(iClient, iSlot, hItem);
            }
            else
            {
                SetItemAtSlot(iClient, iSlot, INVALID_HANDLE);
            }
        }
    }
    else
    {
        for (new iSlot = 0; iSlot < MAX_SLOTS; iSlot++)
        {
            SetItemAtSlot(iClient, iSlot, INVALID_HANDLE);
        }
    }
}

/* WriteClientPrefs()
**
** Writes the client preferences for this user.
** -------------------------------------------------------------------------- */
WriteClientPrefs(iClient, iSlot, Handle:hItem)
{
    new iClass = _:TF2_GetPlayerClass(iClient);
    decl String:strName[64];
            
    if (AreClientCookiesCached(iClient))
    {
        if (hItem == INVALID_HANDLE)
        {
            SetClientCookie(iClient, g_hClientPrefs[iClass][iSlot], "");
        }
        else
        {
            GetItemName(hItem, strName, sizeof(strName));
            SetClientCookie(iClient, g_hClientPrefs[iClass][iSlot], strName);
        }
    }
}

/*
**
**    ______                                          __    
**   / ____/___  ____ ___  ____ ___  ____ _____  ____/ /____
**  / /   / __ \/ __ `__ \/ __ `__ \/ __ `/ __ \/ __  / ___/
** / /___/ /_/ / / / / / / / / / / / /_/ / / / / /_/ (__  ) 
** \____/\____/_/ /_/ /_/_/ /_/ /_/\__,_/_/ /_/\__,_/____/  
**                                                       
**
*/

/* Cmd_BlockTriggers()
**
** Filters the command triggers.
** -------------------------------------------------------------------------- */
public Action:Cmd_BlockTriggers(iClient, iArgs)
{
    if (iClient < 1 || iClient > MaxClients) return Plugin_Continue;
    if (iArgs < 1) return Plugin_Continue;
    
    decl String:strArgument[64];GetCmdArg(1, strArgument, sizeof(strArgument));
    if (StrEqual(strArgument, "!tf_items",      true)) return Plugin_Handled;
    if (StrEqual(strArgument, "!tf_items_menu", true)) return Plugin_Handled;
    if (StrEqual(strArgument, "!tf_items_slot", true)) return Plugin_Handled;
    if (StrEqual(strArgument, "!items",         true)) return Plugin_Handled;
    if (StrEqual(strArgument, "!hats",          true)) return Plugin_Handled;
    if (StrEqual(strArgument, "!em",            true)) return Plugin_Handled;
    return Plugin_Continue;
}

/* Cmd_Items()
**
** Command to display equip/remove items.
** -------------------------------------------------------------------------- */
public Action:Cmd_Items(iClient, iArgs)
{
    if (iClient == 0) { ReplyToCommand(iClient, "[TF2] Command can only be used in-game!"); return Plugin_Handled;          }
    if (iArgs == 0)   { DisplayMenu(g_hMenuMain, iClient, 0); return Plugin_Handled;                                        }
    if (iArgs != 2)   { ReplyToCommand(iClient, "[TF2] Usage: tf_items <equip/remove> <name/slot>"); return Plugin_Handled; }
    
    decl String:strOperation[64]; GetCmdArg(1, strOperation, sizeof(strOperation));
    decl String:strItem[64];      GetCmdArg(2, strItem, sizeof(strItem));
    
    if (StrEqual(strOperation, "equip", false))
    {
        new Handle:hItem = FindItemByName(strItem);
        if (hItem != INVALID_HANDLE) EquipPlayerWearableItem(iClient, hItem);
        else { ReplyToCommand(iClient, "[TF2] Item with name \"%s\" was not found.", strItem); return Plugin_Handled; }
    }
    else if (StrEqual(strOperation, "remove", false))
    {
        if (StrEqual(strItem, "all"))
            for (new iSlot = 0; iSlot < MAX_SLOTS; iSlot++)
                RemoveWearableItemSlot(iClient, iSlot);
        else
        {
            new iSlot = StringToInt(strItem);
            if (iSlot < 0 || iSlot >= MAX_SLOTS)
            {
                ReplyToCommand(iClient, "[TF2] Invalid slot number. Valid slot numbers are between 0 and %i", MAX_SLOTS-1);
                return Plugin_Handled;
            }
            else RemoveWearableItemSlot(iClient, iSlot);
        }
    }
    else ReplyToCommand(iClient, "[TF2] Usage: tf_items <equip/remove> <name/slot>");
    return Plugin_Handled;
}

/*
**
**     __  ___                     
**    /  |/  /__  ____  __  _______
**   / /|_/ / _ \/ __ \/ / / / ___/
**  / /  / /  __/ / / / /_/ (__  ) 
** /_/  /_/\___/_/ /_/\__,_/____/  
**                                 
**
*/

/* MenuCreateMain()
**
** Creates the main menu.
** -------------------------------------------------------------------------- */
Handle:MenuCreateMain()
{
    new iSlotCount = MAX_SLOTS;
    decl String:strKey[64];
    decl String:strSlotName[64];
    decl String:strSlotInfo[64];
    
    // Create menu and setup.
    new Handle:hMenu = CreateMenu(MenuHandler, MENU_ACTIONS_DEFAULT|MenuAction_DisplayItem);
    SetMenuTitle(hMenu, "Item Manager [BETA]");
    SetMenuExitButton(hMenu, true);
    {
        for (new iSlot = 0; iSlot < iSlotCount; iSlot++)
        {
            Format(strKey, sizeof(strKey), "slot%i", iSlot);
            Format(strSlotInfo, sizeof(strSlotInfo), "%i", iSlot);
            if (GetTrieString(g_hGlobal, strKey, strSlotName, sizeof(strSlotName)))
            {
                AddMenuItem(hMenu, strSlotInfo, strSlotName);
            }
        }
    }
    
    return hMenu;
}

/* MenuCreateSlot()
**
** Creates a slot menu.
** -------------------------------------------------------------------------- */
Handle:MenuCreateSlot(iSlot)
{
    decl String:strBuffer[255];
    decl String:strBuffer2[255];
    new iItemCount = CountItems();
    new Handle:hItem;
    
    // Get slot name
    Format(strBuffer, sizeof(strBuffer), "slot%i", iSlot);
    if (!GetTrieString(g_hGlobal, strBuffer, strBuffer2, sizeof(strBuffer2)))
        return INVALID_HANDLE;

    new Handle:hMenu = CreateMenu(MenuHandler, MENU_ACTIONS_DEFAULT|MenuAction_DrawItem);
    SetMenuTitle(hMenu, strBuffer2);
    SetMenuExitButton(hMenu, true);
    SetMenuExitBackButton(hMenu, true);
    {
        Format(strBuffer, sizeof(strBuffer), "%i", iSlot);
        AddMenuItem(hMenu, strBuffer, "[Remove item]");
        for (new iItem = 0; iItem < iItemCount; iItem++)
        {
            hItem = GetItemFromTable(iItem);
            if (GetItemSlot(hItem) != iSlot) continue;
            
            GetItemName(hItem, strBuffer, sizeof(strBuffer));
            GetItemLongName(hItem, strBuffer2, sizeof(strBuffer2));
            AddMenuItem(hMenu, strBuffer, strBuffer2);
        }
    }

    return hMenu;
}

/* MenuDisplayMain()
**
** Displays the main menu for manipulating the items. Very crappy management,
** but does the job for now.
** -------------------------------------------------------------------------- */
public MenuHandler(Handle:hMenu, MenuAction:maAction, iParam1, iParam2)
{
    switch(maAction)
    {
        case MenuAction_Select:
        {
            decl String:strInfo[64];
            GetMenuItem(hMenu, iParam2, strInfo, sizeof(strInfo));
            
            if (hMenu == g_hMenuMain)
            {
                DisplayMenu(g_hMenuSlots[StringToInt(strInfo)], iParam1, 0);
            }
            else
            {                
                if (iParam2 == 0) RemoveWearableItemSlot(iParam1, StringToInt(strInfo));
                else              EquipPlayerWearableItem(iParam1, FindItemByName(strInfo));
                DisplayMenu(g_hMenuMain, iParam1, 0);
            }
        }

        case MenuAction_DrawItem:
        {
            if ((hMenu != g_hMenuMain) && (iParam2 > 0))
            {
                decl String:strInfo[64];
                GetMenuItem(hMenu, iParam2, strInfo, sizeof(strInfo));
                
                if (!QueryPlayerCanUseItem(iParam1, FindItemByName(strInfo)))
                    return ITEMDRAW_IGNORE;
                    //return ITEMDRAW_DISABLED;
            }
            return ITEMDRAW_DEFAULT;
        }
        
        case MenuAction_DisplayItem:
        {
            if (hMenu == g_hMenuMain)
            {
                decl String:strInfo[64];
                decl String:strDisplay[255];
                decl String:strDisplayFinal[255];
                decl String:strItemName[255];
                decl iStyle;
                
                GetMenuItem(hMenu, iParam2, strInfo, sizeof(strInfo), iStyle, strDisplay, sizeof(strDisplay));
                new iSlot = StringToInt(strInfo);
                
                new Handle:hItem = GetItemAtSlot(iParam1, iSlot);
                if (hItem != INVALID_HANDLE)
                {
                    GetItemLongName(hItem, strItemName, sizeof(strItemName));
                    Format(strDisplayFinal, sizeof(strDisplayFinal), "%s [%s]", strDisplay, strItemName);
                    return RedrawMenuItem(strDisplayFinal);
                }
            }
            return 0;
        }
        
        case MenuAction_Cancel:
        {
            if ((iParam2 == MenuCancel_ExitBack) && (hMenu != g_hMenuMain))
            {
                DisplayMenu(g_hMenuMain, iParam1, 0);
            }
        }
    }
    return 0;
}

/*
**
**     __                    ___            
**    / /   ____  ____ _____/ (_)___  ____ _
**   / /   / __ \/ __ `/ __  / / __ \/ __ `/
**  / /___/ /_/ / /_/ / /_/ / / / / / /_/ / 
** /_____/\____/\__,_/\__,_/_/_/ /_/\__, /  
**                                 /____/   
**
*/

/* OnItemPreload()
**
** When the items are about to be precached.
** -------------------------------------------------------------------------- */
public OnItemPreload()
{
    decl String:strRepository[PLATFORM_MAX_PATH];
    GetTrieString(g_hGlobal, "repository", strRepository, sizeof(strRepository));
    ParseDirectory(strRepository);
}

/* ParseConfiguration()
**
** Parses the configuration file.
** -------------------------------------------------------------------------- */
ParseConfiguration()
{
    decl String:strFile[PLATFORM_MAX_PATH];
    BuildPath(Path_SM, strFile, sizeof(strFile), "/configs/items/config.cfg");
    
    if (g_hGlobal == INVALID_HANDLE) g_hGlobal = CreateTrie();
    g_hParsedTrie = g_hGlobal;
    if (ParseSMC(strFile) != Manager_Success) SetFailState("Couldn't PARSE the config file.");
}

/* ParseDirectory()
**
** Parses an item and uploads it to the core plugin.
** -------------------------------------------------------------------------- */
ParseDirectory(String:strDirectory[])
{
    new Handle:hDirectory = OpenDirectory(strDirectory);
    if (hDirectory == INVALID_HANDLE)
    {
        return;
    }
    
    decl String:strFile[PLATFORM_MAX_PATH];
    decl String:strResFile[PLATFORM_MAX_PATH];
    decl FileType:ftType;
    //decl eManagerCode:mcReturn;
    
    while(ReadDirEntry(hDirectory, strFile, sizeof(strFile), ftType))
    {
        if (ftType != FileType_Directory) continue;
        Format(strResFile, sizeof(strResFile), "%s%s/item.res", strDirectory, strFile);
        ParseSMC(strResFile);
    }
}

/* ParseSMC()
**
** Parses an SMC and stores the results into a Trie. Useful for both the
** config file and the actual items.
** -------------------------------------------------------------------------- */
eManagerCode:ParseSMC(String:strFile[])
{    
    decl iLine, SMCError:smcError, String:strError[256];
    
    // Check if the given item exists, in either the internal FS or
    // in the HDD.
    if (!FileExists(strFile, true) && !FileExists(strFile, false)) return ManagerError_FileNotFound;
        
    // Create item structure and the parser.
    if (g_hParser == INVALID_HANDLE)
    {
        g_hParser = SMC_CreateParser();
        SMC_SetReaders(g_hParser, ParseNewSection, ParseKeyValue, ParseEndSection);
    }

    // Parse the file.
    smcError = SMC_ParseFile(g_hParser, strFile, iLine);
    if (smcError != SMCError_Okay)
    {
        SMC_GetErrorString(smcError, strError, sizeof(strError));
        LogError("Could not parse file (line %d, file \"%s\"):", iLine, strFile);
        LogError("Parser encountered error: %s", smcError);
        return ManagerError_ParseError;
    }
    
    return Manager_Success;
}

/* Parse...()
**
** Reads all the key values in the SMC file and stores them into the Trie
** structure.
** -------------------------------------------------------------------------- */
public SMCResult:ParseNewSection(Handle:hSMC, const String:strName[], bool:bOptQuotes)
{
    if (StrEqual(strName, "item"))
    {
        g_hParsedTrie = CreateTrie();
        g_bParsingItem = true;
    }
    return SMCParse_Continue;
}
public SMCResult:ParseEndSection(Handle:hSMC)
{
    if (g_bParsingItem)
    {
        decl String:strItemName[64], Handle:hItem;
        GetItemName(g_hParsedTrie, strItemName, sizeof(strItemName));
        if ((hItem = FindItemByName(strItemName)) != INVALID_HANDLE) DeleteItem(hItem);
        CreateItem(g_hParsedTrie);
    }
    g_bParsingItem = false;
    return SMCParse_Continue;
}
public SMCResult:ParseKeyValue(Handle:hSMC, const String:strKey[], const String:strValue[], bool:bKeyQuotes, bool:bValueQuotes)
{
    // Explode key name string to retrieve type and name.
    decl String:strKeyData[2][64];
    ExplodeString(strKey, ":", strKeyData, 2, 64);
    
    // Determine type
    new iKeyType = -1;
    if (StrEqual(strKeyData[0], "integer")) iKeyType = 0;
    else if (StrEqual(strKeyData[0], "float")) iKeyType = 1;
    else if (StrEqual(strKeyData[0], "string")) iKeyType = 2;
    
    // If no valid type was found on this key, register on log and exit.
    if (iKeyType == -1)
    {
        LogMessage("Invalid type found on key \"%s\".", strKey);
        return SMCParse_Continue;
    }

    // Store key.
    switch(iKeyType)
    {
        case 0: SetTrieValue(g_hParsedTrie, strKeyData[1], StringToInt(strValue), true);
        case 1: SetTrieValue(g_hParsedTrie, strKeyData[1], StringToFloat(strValue), true);
        case 2: SetTrieString(g_hParsedTrie, strKeyData[1], strValue, true);
    }
    
    // Done
    return SMCParse_Continue;
}

/*
**
**   ______            __    
**  /_  __/___  ____  / /____
**   / / / __ \/ __ \/ / ___/
**  / / / /_/ / /_/ / (__  ) 
** /_/  \____/\____/_/____/  
**
**
*/

/* IsValidClient()
**
** Determines if the given client is valid (valid index, connected and in-game).
** -------------------------------------------------------------------------- */
bool:IsValidClient(iClient)
{
    if (iClient < 0 || iClient > MaxClients) return false;
    if (!IsClientConnected(iClient)) return false;
    return IsClientInGame(iClient);
}